library(EBImage)

beerwah = imageData(readImage("beerwah.png"))

x0 = 487834.74
y0 = 7024500.74

dx = 5.06
dy = dx

nx = 256
ny = 203

# Character array to hold each line of the output JS array:
out_lines = character(nx)

for (i in 1:nx) {
  this_line = "["
  x = x0 + (i-1)*dx
  
  for (j in 1:ny) {
    y = y0 + (j-1)*dy
    
    # Conversion from easting/northing to pixel coords; add 1
    # because R uses 1-indexed arrays and I calculated the
    # regressions from 0-indexed pixel co-ordinates.
    px_x =  0.5433962677*x -  264636.468 + 1
    px_y = -0.5462487477*y + 3837812.433 + 1
    
    # That conversion will usually not land precisely on a pixel,
    # so do a bi-linear interpolation.
    px_x1 = floor(px_x)
    px_x2 = px_x1 + 1
    px_y1 = floor(px_y)
    px_y2 = px_y + 1
    
    interp_LHS = t(matrix(
      c(px_x1, px_y1, px_x1*px_y1, 1,
        px_x1, px_y2, px_x1*px_y2, 1,
        px_x2, px_y1, px_x2*px_y1, 1,
        px_x2, px_y2, px_x2*px_y2, 1),
      nrow=4, ncol=4
    ))
    
    # Array to hold R, G, B values:
    this_col = numeric(3)
    
    # Loop over each RGB channel:
    for (k in 1:3) {
      interp_RHS = c(
        beerwah[px_x1, px_y1, k],
        beerwah[px_x1, px_y2, k],
        beerwah[px_x2, px_y1, k],
        beerwah[px_x2, px_y2, k]
      )
      
      coeffs = solve(interp_LHS, interp_RHS)
      
      this_col[k] = round(255*(coeffs[1]*px_x + coeffs[2]*px_y + coeffs[3]*px_x*px_y + coeffs[4]))
    }
    
    # Numerical colour value for use in JS:
    out_col = 65536*this_col[1] + 256*this_col[2] + this_col[3]
    
    comma = ","
    if (j == ny) { comma = ""}
    
    this_line = sprintf("%s%d%s", this_line, out_col, comma)
  }
  
  comma = ","
  if (i == nx) { comma = "\n];"}
  
  this_line = sprintf("%s]%s", this_line, comma)
  
  out_lines[i] = this_line
}

write(out_lines, "colors.txt")
